/* 
-------------------------------------------------------------------------------
/home/duke/davek/YagiUda-1.0/num_elements.c
-------------------------------------------------------------------------------

This file is part of a set of source files for a suite of programs
for the analysis and optimisation of Yagi-Uda antennas. 

Name: YagiUda (executables are: 'input', 'yagi', 'output' and 'optimise' )

Written by: David Kirkby BSc MSc G8WRB (email:davek@medphys.ucl.ac.uk)

Version: 1.00

The complete distribution should be found in a file YagiUda-1.0.tar.Z, 
which will be found on ftp sites. Please e-mail me if you have problems
locating the latest version.

*/

#include <stdio.h>
#include <string.h>
#include "yagi.h"

int get_number_of_elements(char *input_filename, int *driven , int *parasitic)
{
	FILE *ifp;
	char *null, *line;
	int num_elements;

	null=string(0,12);
	line=string(0,81);
	ifp=fopen(input_filename, "r");
	*driven=-1;
	*parasitic=-1;
	if(ifp == NULL)
	{
		fprintf(stderr,"Sorry, cant find file:  %s\n", input_filename);
		exit(2);
	}
	/* Read line by line, looking for data on number of elements */
	while(!feof(ifp))
	{
		fgets(line, 80, ifp);
		if(strncmp(line,"ELEMENTS",8) == 0)  
		{
			sscanf(line,"%s %d\n", null, &num_elements);						
		}
		if(strncmp(line,"DRIVEN",6) == 0)  
		{
			sscanf(line,"%s %d\n", null, driven);						
		}
		if(strncmp(line,"PARASITIC",8) == 0)  
		{
			sscanf(line,"%s %d\n", null, parasitic);						
		}
	}
	fclose(ifp);
	if(num_elements == -1)
		fprintf(stderr,"Error in data file %s: ELEMENTS undefined\n",input_filename); 
	if(*driven == -1)
		fprintf(stderr,"Error in data file %s : DRIVEN undefined\n", input_filename); 
	if(*parasitic == -1)
		fprintf(stderr,"Error in data file %s: PARASITIC undefined\n", input_filename); 
	if(num_elements != *driven + *parasitic)
	{
		fprintf(stderr,"Check ELEMENTS, DRIVEN & PARASITIC in data file %s\n", input_filename);
		fprintf(stderr,"ELEMENTS = %d DRIVEN = %d PARASITIC = %d\n", num_elements,      *driven, *parasitic);
		exit(3);
	}
	free_string(line,0,81);
	return(num_elements);
}

