/* 
-------------------------------------------------------------------------------
/home/duke/davek/YagiUda-1.0/quick.c
-------------------------------------------------------------------------------

This file is part of a set of source files for a suite of programs
for the analysis and optimisation of Yagi-Uda antennas. 

Name: YagiUda (executables are: 'input', 'yagi', 'output' and 'optimise' )

Written by: David Kirkby BSc MSc G8WRB (email:davek@medphys.ucl.ac.uk)

Version: 1.00

The complete distribution should be found in a file YagiUda-1.0.tar.Z, 
which will be found on ftp sites. Please e-mail me if you have problems
locating the latest version.

*/

#include <stdio.h>
#include <malloc.h>
#include <math.h>
#include <sys/types.h>
#include <string.h>
#include "yagi.h"

/* This program opens an input file, then calls yagi to calculate parameters
at just one frequency. It then look at the .dat file to find both gain and FB 
ratio. It then randomly shifts the elements a bit (both length and position)
then recalculates again. If its better, we have a better yagi. If so, new 
positions are copied to the input file. This continues forever ... */

int main(int argc, char **argv)
{

	FILE *fp, *fp_dat, *fp_tmp, *fp_stop, *update_fp;
	int elements, driven, parasitic, itterations;
	int randomisation, choice, i;
	double **driven_data, **parasitic_data, angular_step, scale_factor=1.0;
	double frequency, min_frequency, max_frequency, step_frequency;
	char *input_filename, *dot_dat_filename, *dot_tmp_filename, *notes="tmp";
	char *do_yagi, *do_output, *copy_to_tmp, *copy_backup_from_tmp, *line;
	double old_gain, old_fb_ratio, new_gain, new_fb_ratio, percent;
	double new_X, old_X, old_R, new_R;
	char *dot_tmp_dot_dat_filename;
	long seed;

	if(argc!=6)
	{
		fprintf(stderr, "USAGE: %s  filename percent randomise better iterations.\n", argv[0]);
		fprintf(stderr,"Where: randomise; 0=all, 1=driven, 2=parasitics\n");
		fprintf(stderr,"     : better; 0=everthing, 1=G, 2=FB, 3=G&FB, 4=resonate,5=R,G&FB, 6=R, 7=R&G\n");
		exit(1);
	}
	percent=atof(argv[2]);
	randomisation=atoi(argv[3]);
	choice=atoi(argv[4]);
	itterations=atoi(argv[5]);

	time(&seed);
	srand48(seed);
	update_fp=fopen("update","w"); /* delete */
	fclose(update_fp);
	/* allocate memory for strings */

	line=string(0L,1000L);
	input_filename=string(0L,100L);
	dot_dat_filename=string(0L, 100L);
	dot_tmp_filename=string(0L, 100L);
	dot_tmp_dot_dat_filename=string(0L, 100L);
	copy_to_tmp=string(0L, 100L);
	copy_backup_from_tmp=string(0L, 100L);
	do_yagi=string(0L, 100L);
	do_output=string(0L, 100L);
	/* We here get all the file names, then open files. */
	strcpy(input_filename, *(argv+1));
	strcpy(dot_dat_filename, *(argv+1));
	strcpy(dot_tmp_dot_dat_filename, *(argv+1));
	strcat(dot_dat_filename,".dat");
	strcat(dot_tmp_dot_dat_filename,".tmp");
	strcat(dot_tmp_dot_dat_filename,".dat");
	strcpy(dot_tmp_filename, *(argv+1));
	strcat(dot_tmp_filename,".tmp");
	fp=fopen(input_filename,"r");
	fp_tmp=fopen(dot_tmp_filename,"w");
	if(fp == NULL)
	{
		fprintf(stderr,"Cant open %s\n", input_filename);
		exit(10);
	}
	strcpy(do_yagi,"yagi ");
	strcat(do_yagi,dot_tmp_filename);
	strcpy(do_output,"output ");
	strcat(do_output,dot_tmp_filename);
	strcpy(copy_to_tmp,"cp ");
	strcat(copy_to_tmp,dot_tmp_filename);
	strcat(copy_to_tmp," tmp");
	strcpy(copy_backup_from_tmp,"cp tmp ");
	strcat(copy_backup_from_tmp,dot_tmp_filename);
	/* since the size of all the arrays depends on the number of elements,
	lets first find out how many elements we have, including driven and 
	parasitic sepparately.  Then try to allocate all the memory we need. If we 
	fail, then not much time is wasted. */
 	elements=get_number_of_elements(input_filename, &driven, &parasitic);
	/* allocate all memory */
	driven_data=dmatrix(1L,(long)(driven),1L,6L); 
	parasitic_data=dmatrix(1L,(long) (parasitic),1L,4L); 
	read_yagi_data(line, input_filename, &frequency, &min_frequency, &max_frequency,      &step_frequency, driven, driven_data, parasitic, parasitic_data,                &angular_step);
	/* Only work at one frequency, so force this */
	frequency/=1000000;
	min_frequency=frequency;
	max_frequency=frequency+0.01;
	step_frequency=0.1; 
	/* wont use pattern, so dont wast time calculate any finer than necessary */
	angular_step=180.0;		
	/* write to fred.tmp or whatever */
	write_input_data_to_disk(fp_tmp, notes, frequency, min_frequency,               max_frequency, step_frequency, elements, driven, parasitic, angular_step,       driven_data, parasitic_data, scale_factor); 
	fclose(fp_tmp);
	for(i=1; i<=itterations; ++i)
	{
		fill_z_matrix();	
		randomise(randomisation, frequency, percent, driven_data, parasitic_data, driven, parasitic);

	}
	fclose(fp);
	fclose(fp_dat);
}
