/* 
-------------------------------------------------------------------------------
/home/duke/davek/YagiUda-1.0/randomise.c
-------------------------------------------------------------------------------

This file is part of a set of source files for a suite of programs
for the analysis and optimisation of Yagi-Uda antennas. 

Name: YagiUda (executables are: 'input', 'yagi', 'output' and 'optimise' )

Written by: David Kirkby BSc MSc G8WRB (email:davek@medphys.ucl.ac.uk)

Version: 1.00

The complete distribution should be found in a file YagiUda-1.0.tar.Z, 
which will be found on ftp sites. Please e-mail me if you have problems
locating the latest version.

*/

#include <stdio.h>
#include <math.h>
#ifdef i386
#include <random.h>
#endif
#ifdef sun
#include <stdlib.h>
#endif
#include "yagi.h"

void randomise(int randomisation_method, double frequency, double max_percent, double **driven_data, double**parasitic_data, int driven, int parasite)
{
	int i;
	static int run_first_time=TRUE;
	double y;
	static double max_boom_length;

	if(run_first_time==TRUE)
	{
		run_first_time=FALSE;
		max_boom_length=1.10*parasitic_data[parasite][X];
	}
	if(randomisation_method == ALL)
	{
		for(i=1;i<=driven; ++i) /* randomise the driven */
		{
			/* adjust length only of driven element */
			y=1+0.01*max_percent*(drand48()-0.5);
			driven_data[i][LENGTH]*=y;
			if(driven_data[i][LENGTH] > (150.0/frequency))
				driven_data[i][LENGTH]=(150.0/frequency); 
		}
		/* do first parasite (reflector). */
		y=1+0.01*max_percent*(drand48()-0.5);
		parasitic_data[1][X]*=y;
		y=1+0.01*max_percent*(drand48()-0.5);
		parasitic_data[1][LENGTH]*=y; 

		/* do other parasites (directors ) */
		for(i=2; i<parasite; ++i) 
		{
			y=1+0.01*max_percent*(drand48()-0.5);
			parasitic_data[i][X]*=y;
			y=1+0.01*max_percent*(drand48()-0.5);
			parasitic_data[i][LENGTH]*=y;
			if(parasitic_data[i][LENGTH] > (150.0/frequency)) 
				parasitic_data[i][LENGTH]=150.0/frequency; 
		} 
		/* Position last element, keeping it within 10% of it original 
		position */
		y=1+0.01*max_percent*(drand48()-0.5);
		parasitic_data[parasite][X]*=y;
		if(parasitic_data[parasite][X]>max_boom_length)
			parasitic_data[parasite][X]=max_boom_length;
		y=1+0.01*max_percent*(drand48()-0.5);
		parasitic_data[parasite][LENGTH]*=y;
		if(parasitic_data[parasite][LENGTH] > (150.0/frequency)) 
			parasitic_data[parasite][LENGTH]=150.0/frequency; 
	}
	else if (randomisation_method == DRIVEN_ONLY)
	{
		for(i=1;i<=driven; ++i) /* randomise the driven */
		{
			/* adjust length only of driven element */
			y=1+0.01*max_percent*(drand48()-0.5);
			driven_data[i][LENGTH]*=y;
		}
	}
	else if (randomisation_method == PARASITES_ONLY)
	{
		/* do first parasite (reflector). */
		y=1+0.01*max_percent*(drand48()-0.5);
		parasitic_data[1][X]*=y;
		y=1+0.01*max_percent*(drand48()-0.5);
		parasitic_data[1][LENGTH]*=y; 

		/* do other parasites (directors ) */
		for(i=2; i<parasite; ++i) 
		{
			y=1+0.01*max_percent*(drand48()-0.5);
			parasitic_data[i][X]*=y;
			y=1+0.01*max_percent*(drand48()-0.5);
			parasitic_data[i][LENGTH]*=y;
			if(parasitic_data[i][LENGTH] > (150.0/frequency)) 
				parasitic_data[i][LENGTH]=150.0/frequency; 
		} 
		/* Position last element, kepping it within 10% of it original 
		position */
		y=1+0.01*max_percent*(drand48()-0.5);
		parasitic_data[parasite][X]*=y;
		if(parasitic_data[parasite][X]>max_boom_length)
			parasitic_data[parasite][X]=max_boom_length;
		y=1+0.01*max_percent*(drand48()-0.5);
		parasitic_data[parasite][LENGTH]*=y;
		if(parasitic_data[parasite][LENGTH] > (150.0/frequency)) 
			parasitic_data[parasite][LENGTH]=150.0/frequency; 
	}
}
