/* 
-------------------------------------------------------------------------------
/home/duke/davek/YagiUda-1.0/string.c
-------------------------------------------------------------------------------

This file is part of a set of source files for a suite of programs
for the analysis and optimisation of Yagi-Uda antennas. 

Name: YagiUda (executables are: 'input', 'yagi', 'output' and 'optimise' )

Written by: David Kirkby BSc MSc G8WRB (email:davek@medphys.ucl.ac.uk)

Version: 1.00

The complete distribution should be found in a file YagiUda-1.0.tar.Z, 
which will be found on ftp sites. Please e-mail me if you have problems
locating the latest version.

*/

#include <stdio.h>
#include <malloc.h>
#include "yagi.h"


/* The authors of the book 'Numerical Recipes in C' were kind enough
to make the utility routines in the files 'nrutil.c' and 'nrutil.h'
public domain. Hence I can include both without any problems */


#define NR_END 1
#define FREE_ARG char*

char *string(long nl,long nh)
{
	char *v;

	v=(char *)malloc((unsigned) (nh-nl+1)*sizeof(char));
	if (!v) nrerror("allocation failure in string()");
	return v-nl;
}

void free_string(char *v, long nl, long nh)
/* free a string allocated with string() */
{
	/* free((FREE_ARG) (v+nl-NR_END)); */
	free((FREE_ARG) (v+nl));
}

FCOMPLEX **FCOMPLEXmatrix(long nrl, long nrh, long ncl, long nch)
/* allocate a FCOMPLEX matrix with subscript range m[nrl..nrh][ncl..nch] */
{
	long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	FCOMPLEX **m;

	/* allocate pointers to rows */
	m=(FCOMPLEX **) malloc((size_t)((nrow+NR_END)*sizeof(FCOMPLEX*)));
	if (!m) nrerror("allocation failure 1 in matrix()");
	m += NR_END;
	m -= nrl;

	/* allocate rows and set pointers to them */
	m[nrl]=(FCOMPLEX *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(FCOMPLEX)));
	if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
	m[nrl] += NR_END;
	m[nrl] -= ncl;

	for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	/* return pointer to array of pointers to rows */
	return m;
}

element_data *element_data_vector(long nl, long nh)
/* allocate an element_data vector with subscript range v[nl..nh] */
{
	 element_data *v;

	v=(element_data  *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(element_data)));
	if (!v) nrerror("allocation failure in element_data_vector()");
	return v-nl+NR_END;
}

FCOMPLEX *FCOMPLEXvector(long nl, long nh)
/* allocate a FCOMPLEX vector with subscript range v[nl..nh] */
{
	FCOMPLEX *v;

	v=(FCOMPLEX *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(FCOMPLEX)));
	if (!v) nrerror("allocation failure in FCOMPLEXvector()");
	return v-nl+NR_END;
}
