/* 
-------------------------------------------------------------------------------
/home/duke/davek/YagiUda-1.0/yagi.h
-------------------------------------------------------------------------------

This file is part of a set of source files for a suite of programs
for the analysis and optimisation of Yagi-Uda antennas. 

Name: YagiUda (executables are: 'input', 'yagi', 'output' and 'optimise' )

Written by: David Kirkby BSc MSc G8WRB (email:davek@medphys.ucl.ac.uk)

Version: 1.00

The complete distribution should be found in a file YagiUda-1.0.tar.Z, 
which will be found on ftp sites. Please e-mail me if you have problems
locating the latest version.

*/

#include <stdio.h>
#include "nrutil.h"
#include "complex_hack.h"
#include "nr_hack.h"

#define TRUE 1
#define FALSE 0

#define MAX_LINE  100000  /* Maximum characters on one line of input file */

#define ALL 0               /* randomisation method in 'optimise' */
#define DRIVEN_ONLY 1
#define PARASITES_ONLY 2

#define EVERYTHING 0       /* better criteria, see optimise and better.c */
#define GAIN 1
#define FB_RATIO 2
#define GAIN_AND_FB_RATIO 3
#define RESONANCE      4
#define RESISTANCE_AND_GAIN_AND_FB 5
#define RESISTANCE 6
#define RESISTANCE_AND_GAIN 7


typedef struct element_data {double x,y,length;};

int main(int argc, char **argv);
char *get_data_filenames(int argc, char **argv, char *input);
char *string(long lower, long upper);
void free_string(char *string, long lower, long upper);
int get_number_of_elements(char *datafile, int *driven, int *parasitic);
void read_yagi_data(char *line, char *file, double *frequency, double *min_frequency, double*max_frequency, double *step_frequency, int dr, double **dr_data, int parasitic, double **para_data, double *angular_step);
void   self_impedance(int i, double frequency, int driven, int parasitic, double **data, double **impedance);
void   mutual_impedance(int i, int j, double frequency, int driven, int parasitic, double **d, double **p, double **impedance);
void   fill_z_matrix(double frequency, int driven, int parasitic, double **d, double **p, double **impedance);
void z21(double lamda, double d, double mean_length, double *r21, double *x21);
double ci(double); 
void fill_v_vector(int driven, int parasitic, double **d, double *v_vector);
void write_header_to_disk(FILE *ofp, int elements, int driven,
int parasitic, double min_frequency, double max_frequency, double frequency,
double step_frequency, double angular_step);
void write_vector(double *v, int elements, FILE *ofp);
int  read_header(FILE *ifp, FILE *ofp, double *min_f, double *max_f, double *step_f, double *f, double *angular_step);
void z_input(FCOMPLEX v, FCOMPLEX i, FCOMPLEX *z);
void reflection_coefficient(FCOMPLEX z_input, double *mag, double *phase);
double calculate_vswr(double magnitude);
double calculate_power_input(double real_z, FCOMPLEX current);
void   write_coordinates_of_elements_to_disk(FILE *ofp, int driven,
int parasitic, double **driven_data, double **parasitic_data);
void gain(double theta, double phi, double  pin, double normallised_f, element_data *cordinates,FCOMPLEX *current, int elements, double *E, double *H, double actual_frequency, double design_frequency);
void read_performance(char *file, double *gain, double *fb_ratio, double *reactance, double *resistance);
FCOMPLEX E_to_complex_power(FCOMPLEX x);
FCOMPLEX **FCOMPLEXmatrix(long a, long b, long c, long d);
FCOMPLEX *FCOMPLEXvector(long a, long b);
element_data *element_data_vector(long low, long high);
void print_z_matrix(double f, int elements, double **z_matrix);
void write_gain_at_various_angles(FILE *gain_fp, double angular_step, double pin, double normalised_f, double f, struct element_data *coordinates, struct FCOMPLEX *current, int elements, double design_f);

void write_input_data_to_disk(FILE *fp, char *notes, double frequency, double min_f, double max_f, double step_f, int elements ,int driven,int parasitic, double angular_step,double **d, double **p, double scale_factor); 
void randomise(int randomisation_method, double frequency, double percent, double **driven_data, double **parasitic_data, int driven, int parasitic);
int is_it_better(int criteria, double new_gain, double new_fb_ratio, double old_gain, double old_fb_ratio, double newX, double oldX, double new_R, double old_R); 
double version(void);

#define MAX_DRIVEN 6			/* There are 6 pieces of info on a driven ele */
#define MAX_PARASITIC 4		/* But only 4 on a parasitic */

#define  X 			1			/* index into driven and parasitic arrays */
#define  Y 			2
#define  LENGTH	3
#define	DIAMETER	4
#define	VOLTAGE	5			/* Volt and phase not included on parasistic ele */
#define	PHASE		6
#define  Z0			50			/* characterisitc impedance to aim for */
#define 	REAL			0
#define 	IMAGINARY	1
#define HEADER_SIZE  100   /* size of header in .out files in bytes */

